#include "rational.hpp" // szksges a fejlcfjl hasznlata
using namespace std; // itt adjuk meg a nvtrhasznlatot

void Rational::simplify(int& a, int& b) const // egyszersts
{
    int c = a, d = b; // euklideszi algoritmus futtatsa
    while (c != d)
    {
        if (c > d)
            c -= d;
        else
            d -= c;
    }
    a /= c;
    b /= d;
}

// konstruktorok:
Rational::Rational()   // 0 paramteres konstruktor
{
    _numerator = 0;
    _denominator = 1;
}
Rational::Rational(int nr)   // 1 paramteres
{
    _numerator = nr;
    _denominator = 1;
}
Rational::Rational(int num, int denom)   // 2 paramteres
{
    _numerator = num;
    _denominator = denom != 0 ? denom : 1;
    simplify(_numerator, _denominator);
}

// lekrdez s bellt mveletek:
void Rational::setNumerator(int value)
{
    _numerator = value;
    simplify(_numerator, _denominator);
}
void Rational::setDenominator(int value)
{
    _denominator = value == 0 ? 1 : value; // specilis bellts
    simplify(_numerator, _denominator);
}
int Rational::numerator() const
{
    return _numerator;
}
int Rational::denominator() const
{
    return _denominator;
}

// egyszer opertorok metdusknt:
Rational Rational::operator+(const Rational& r) const
{
    // sszeads kt racionlis szm kztt
    Rational res;
    res._numerator = _numerator * r._denominator + r._numerator * _denominator;
    res._denominator = _denominator * r._denominator;
    simplify(res._numerator, res._denominator);
    return res;
}

Rational Rational::operator+(int e) const
{
    // sszeads racionlis szm s egsz kztt
    Rational res;
    res._numerator = _numerator + e * _denominator;
    res._denominator = _denominator;
    simplify(res._numerator, res._denominator);
    return res;
}

Rational Rational::operator*(const Rational& r) const
{
    // szorzs kt racionlis szm kztt
    Rational res; // eredmny vltoz
    res._numerator = _numerator * r._numerator;
    res._denominator = _denominator * r._denominator;
    // az eredmnybe vgezzk el a mveleteket
    simplify(res._numerator, res._denominator);
    return res; // az eredmnyt adjuk vissza
}

Rational Rational::operator*(int e) const
{
    // szorzs racionlis szm s egsz kztt
    Rational res;
    res._numerator = _numerator * e;
    res._denominator = _denominator;
    simplify(res._numerator, res._denominator);
    return res;
}

// kls egyszer opertorok:
Rational operator+(int e, const Rational& r)
{
    return r + e; // hasznljuk a mr ksz vltozatot
}

Rational operator*(int e, const Rational& r)
{
    return r * e; // hasznljuk a mr ksz vltozatot
}

// adattviteli, kls opertorok:
istream& operator>>(istream& s, Rational &r)
{
    // s lesz az adatfolyam
    s >> r._numerator >> r._denominator;
    // s-rl beolvassuk a kt rtket
    r.simplify(r._numerator, r._denominator); // egyszersts meghvsa
    return s; // s-t visszaadjuk
}

ostream& operator<<(ostream &s, const Rational& r)
{
    s << r._numerator << "/" << r._denominator;
    // s-re kirjuk a megfelel alakot
    return s;
}

// kls rtkmdostsi opertorok:
Rational operator+=(Rational& r1, const Rational& r2)
{
    r1._numerator = r1._numerator * r2._denominator + r2._numerator * r1._denominator; // a mveletet a rejtett mezkn vgezzk el
    r1._denominator = r1._denominator * r2._denominator;
    r1.simplify(r1._numerator, r1._denominator); // rejtett egyszersts meghvsa
    return r1; // az els paramtert adjuk vissza
}

Rational operator+=(Rational& r, int e)
{
    r._numerator = r. _numerator + e * r._denominator;
    r.simplify(r._numerator, r._denominator);
    return r; // hasznljuk a mr ksz vltozatot
}

int operator+=(int& e, const Rational& r)
{
    e = e + r._numerator / r._denominator; // az egsz szmot kell mdostanunk
    return e; // s azt visszaadnunk
}

Rational operator*=(Rational& r1, const Rational& r2)
{
    r1._numerator *= r2._numerator; // a mveletet a paramteren vgezzk el
    r1._denominator *= r2._denominator;
    r1.simplify(r1._numerator, r1._denominator);
    return r1; // az els paramtert adjuk vissza
}

Rational operator*=(Rational& r, int e)
{
    r._numerator *= e;
    r.simplify(r._numerator, r._denominator);
    return r; // hasznljuk a mr ksz vltozatot
}

int operator*=(int& e, const Rational& r)
{
    e = e * r._numerator / r._denominator; // az egsz szmot kell mdostanunk
    return e; // s azt visszaadnunk
}
